@@ -8,7 +8,7 @@ use Net::OAuth;
 use URI;
 use URI::Escape;
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 my $request_token_url = 'https://api.dropbox.com/1/oauth/request_token';
 my $access_token_url = 'https://api.dropbox.com/1/oauth/access_token';
@@ -38,12 +38,16 @@ sub import {
         require Furl::HTTP;
         require IO::Socket::SSL;
     };if ($@) {
-        require LWP::UserAgent;
-        require HTTP::Request;
-        $WebService::Dropbox::USE_LWP++;
+        __PACKAGE__->use_lwp;
     }
 }
 
+sub use_lwp {
+    require LWP::UserAgent;
+    require HTTP::Request;
+    $WebService::Dropbox::USE_LWP++;
+}
+
 sub new {
     my ($class, $args) = @_;
 
@@ -441,6 +445,9 @@ sub api_lwp {
         my $content_length = $end_pos - $cur_pos;
         push @$headers, 'Content-Length' => $content_length;
     }
+    if ($args->{headers}) {
+        push @$headers, @{ $args->{headers} };
+    }
     my $req = HTTP::Request->new($args->{method}, $args->{url}, $headers, $args->{content});
     my $ua = LWP::UserAgent->new;
     $ua->timeout($self->timeout);
@@ -662,7 +669,7 @@ L<https://www.dropbox.com/developers/start/core>
 
 L<https://www.dropbox.com/developers/reference/api#account-info>
 
-=head2 files(path, output, [params]) - download (no file list, file list is metadata)
+=head2 files(path, output, [params, opts]) - download (no file list, file list is metadata)
 
     # Current Rev
     my $fh_get = IO::File->new('some file', '>');
@@ -672,7 +679,18 @@ L<https://www.dropbox.com/developers/reference/api#account-info>
     # Specified Rev
     $dropbox->files('folder/file.txt', $fh_get, { rev => ... }) or die $dropbox->error;
 
-    # output is fh or code ref.
+    # Code ref
+    $dropbox->files('folder/file.txt', sub {
+        # compatible with LWP::UserAgent and Furl::HTTP
+        my $chunk = @_ == 4 ? @_[3] : $_[0];
+        print $chunk;
+    }) or die $dropbox->error;
+
+    # Range
+    $dropbox->files('folder/file.txt', $fh_get) or die $dropbox->error;
+    > "0123456789"
+    $dropbox->files('folder/file.txt', $fh_get, undef, { headers => ['Range' => 'bytes=5-6'] }) or die $dropbox->error;
+    > "56"
 
 L<https://www.dropbox.com/developers/reference/api#files-GET>
 
